(*===========================================================*)
(* 128x64 pixels graphics demo program                       *)
(* Mainly intended to show how such a display can be         *)
(* controlled via the I2C/TWI BUS                            *)
(* The drawing speed is lower compared with the port mode    *)
(* but it is not much slower                                 *)
(* The second purpose is to show that the HD61202 controller *)
(* is compatible with the KS0108                             *)
(*===========================================================*)

program GraphTWI;

{ $BOOTRST $01E00}         {Reset Jump to $01E00}
{$NOSHADOW}
{ $W+ Warnings}            {Warnings off}

Device = mega128, VCC = 5;

Import SysTick, SerPort, TWImaster, LCDGraphic, BeepPort;

From System Import;
From LCDGraphic Import CharSet;     {block CharSet, pixels}

Define
  ProcClock      = 16000000;        {Hertz}
  SysTick        = 10;              {msec}
  StackSize      = $0024, iData;
  FrameSize      = $0040, iData;
  SerPort        = 19200, Stop2;     {Baud, StopBits|Parity}
  RxBuffer       = 8, iData;
  TxBuffer       = 8, iData;
  BeepPort       = PortG, 4;
  
  TWIpresc       = 10;
  LCDGraphic     = 128, 64, 8;      {x-pix, y-pix, accesswidth}
  // use the Column mode controller 61202 or KS0108 with 2 chipselects
  LCDgraphMode   = column, iData;   // linear
  DefCharSet     = 'Graphchars.pchr';  {FileName}
  GViewports     = 1, iData;        {logical ViewPorts, scalings}
  TGraphStr      = 16;              {Graphic Text String Length}
  

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }
type
  tLCDctrls      = (LCD_EN, LCD_RW, LCD_RS, LCD_CS1, LCD_CS2, LCD_RES, LCD_OUTP, LCD_INP);
  tLCDctrlSet    = bitset of tLCDctrls;

{--------------------------------------------------------------}
const
  PCF9555        : byte = $27;
  LCD_busy       : byte = 7;                           {busy bit}
  LCD_DatRD      : byte = 0;
  LCD_DatWR      : byte = 2;
  LCD_DatDDR     : byte = 6;
  LCD_CtrlRD     : byte = 1;
  LCD_CtrlWR     : byte = 3;
  LCD_CtrlDDR    : byte = 7;

  mybitmap       : array[1..(32*32 div 8) + 2] of byte = 'test1.pbmp';

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var
  CtrlPortSave            : tLCDctrlSet;
  LCD_CS12                : tLCDctrlSet;
  i                       : integer;
  col                     : Byte;                 {column counter}
  CurRow                  : Byte;                 {storage for 2. controller}
  
{--------------------------------------------------------------}
{ functions }
// The busy function is only shown for information
// It is not necessary because we are slower as the
// graphics controller
Procedure LCDbusy;
var busy : byte;
begin
  TWIout(PCF9555, LCD_DatDDR, $FF);        // all pins input
  CtrlPortSave:= LCD_CS12 + [LCD_RW];
  TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);
  Repeat
    CtrlPortSave:= LCD_CS12 + [LCD_RW, LCD_EN];
    TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);
    TWIout(PCF9555, LCD_DatRD);
    TWIinp(PCF9555, busy);
    CtrlPortSave:= LCD_CS12 + [LCD_RW];
    TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);
  until (busy and LCD_busy) = 0;
end;

Procedure WriteLCD( b : Byte; code : Boolean);
begin
// we don't need the busy polling because we are somewhat slow :-)
//  LCDbusy;
  if code then
    CtrlPortSave:= LCD_CS12 + [LCD_RS];
  else
    CtrlPortSave:= LCD_CS12;
  endif;
  TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);
// because we don't use busy the dataport is always in output mode
//  TWIout(PCF9555, LCD_DatDDR, $00);        // all output
  TWIout(PCF9555, LCD_DatWR, b);             // write data

  incl(CtrlPortSave, [LCD_EN]);                // enable strobe
  TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);
  excl(CtrlPortSave, [LCD_EN]);
  TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);
end;

Procedure GraphInit;
begin
  TWIout(PCF9555, LCD_DatDDR, $00);        // all output
  {write init values controller 2}
  LCD_CS12:= [LCD_CS2, LCD_RES];
  TWIout(PCF9555, LCD_CtrlWR, LCD_CS12);
  sDelay(10);
  WriteLCD(%00111111, false);
  WriteLCD(%11000000, false);
  WriteLCD(%10111000, false);
  WriteLCD(%01000000, false);
  {write init values controller 1}
  LCD_CS12:= [LCD_CS1, LCD_RES];
  TWIout(PCF9555, LCD_CtrlWR, LCD_CS12);
  sDelay(10);
  WriteLCD(%00111111, false);
  WriteLCD(%11000000, false);
  WriteLCD(%10111000, false);
  WriteLCD(%01000000, false);
  {clear display}
  gclrscr(0);
  gdisprefresh;
end;

{$D-}
UserDevice GraphIOS(cmd : byte; arg : byte);
begin
  (* Display Refresh with "GraphIOS" *)
  (* cmd = 0   arg = rowaddr    set rowaddr to arg *)
  (* cmd = 1   arg = pixels     write data byte    *)
  if cmd = 0 then
    // set row addr
    CurRow:= arg;
    LCD_CS12:= [LCD_CS1, LCD_RES];
    TWIout(PCF9555, LCD_CtrlWR, LCD_CS12);
    WriteLCD(%10111000 + CurRow, false);
    WriteLCD(%01000000, false);
    col:= 0;
  else
    if col = 64 then
      {if column = 64 then switch to controller 2}
      {and set start values for controller 2}
      LCD_CS12:= [LCD_CS2, LCD_RES];
      TWIout(PCF9555, LCD_CtrlWR, LCD_CS12);
      WriteLCD(%10111000 + CurRow, false);
      WriteLCD(%01000000, false);
      WriteLCD(arg, true);
      inc(col);
    else
      WriteLCD(arg, true);
      inc(col);
    endif;
  endif;
end;
{$D+}

procedure Init9555;
begin
  TWIout(PCF9555, LCD_DatWR, 0);           // all pins null
  TWIout(PCF9555, LCD_DatDDR, $FF);        // all pins input
  TWIout(PCF9555, LCD_CtrlWR, 0);          // all pins null
  TWIout(PCF9555, LCD_CtrlDDR, 0);         // all pins output
  mDelay(10);
  CtrlPortSave:= [LCD_RES];
  TWIout(PCF9555, LCD_CtrlWR, CtrlPortSave);  // remove reset
end;

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}
begin
  EnableInts;
  BeepOutHL;
  mdelay(500);
  if TWIstat(PCF9555) then
    WriteLn(SerOut, 'PCF found');
  else
    WriteLn(SerOut, 'PCF not found');
    loop
      BeepOut(1500, 5);
      mdelay(1000);
    endloop;
  endif;
  Init9555;

  mdelay(100);
  GraphInit;
  TWBR:= 2;
  
  gDrawRect(15, 6, 110, 56, $FF);
  gdisprefresh;
  mdelay(1000);
  gSetTextJustify(alHorCenter, alVertBottom);
  gSetTextTransp(true);
  gSetTextMode(wmSetPix);
  gDrawString(35, 53, 1, 1, TxtRot0, 'Test');
  gSetTextMode(wmXorPix);
  gDrawString(60, 33, 3, 3, TxtRot0, 'E-LAB');
  gDrawString(60, 42, 1, 1, TxtRot0, 'Computers');
  gDrawString(122, 30, 2, 1, TxtRot90, 'AVRco');
  gdisprefresh;
  gdrawbitmap(0, 0, @mybitmap, wmsetpix);
  gdisprefresh;
  mdelay(10);
  loop
    for i:= 1 to 45 do
      gdrawbitmap(0, i - 1, @mybitmap, wmclrpix);
      gDrawString(34 + i, 53, 1, 1, TxtRot0, 'Test');
      gdrawbitmap(0, i, @mybitmap, wmsetpix);
      gDrawString(35 + i, 53, 1, 1, TxtRot0, 'Test');
      gdisprefresh;
    endfor;
    for i:= 44 downto 0 do
      gdrawbitmap(0, i + 1, @mybitmap, wmclrpix);
      gDrawString(36 + i, 53, 1, 1, TxtRot0, 'Test');
      gdrawbitmap(0, i, @mybitmap, wmsetpix);
      gDrawString(35 + i, 53, 1, 1, TxtRot0, 'Test');
      gdisprefresh;
    endfor;
  endloop;
end GraphTWI.

